Added 863 files from NVIDIA/cccl sparse checkout: - c2h/ (27 files): Catch2 test helpers — generators, validators, runner - nvbench_helper/ (10 files): Benchmark harness utilities - cmake/ (29 files): CMake presets and build helpers - cudax/ (794 files): Experimental CUDA extensions - AGENTS.md: NVIDIA's official AI agent instructions for CCCL - CMakePresets.json: Standardized build configurations - cccl-version.json: Version tracking Also added CCCL_ASSET_MAP.md mapping all 4295 CCCL files to competition value and PRD items. cccl_upstream now covers 100% of competition-critical assets: - 27 tuning headers (SM80/90/100 benchmark data) - 32 dispatch headers (algorithm implementations) - 60 Thrust examples (correctness verification) - 217 CUB Catch2 tests (regression matrix) - 153 CUB benchmarks (parameter space search) - 18 CUB examples (API verification) - 27 test helpers + benchmark harness - 794 cudax experimental extensions
79 lines
2.1 KiB
Plaintext
79 lines
2.1 KiB
Plaintext
//===----------------------------------------------------------------------===//
|
|
//
|
|
// Part of CUDASTF in CUDA C++ Core Libraries,
|
|
// under the Apache License v2.0 with LLVM Exceptions.
|
|
// See https://llvm.org/LICENSE.txt for license information.
|
|
// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
|
|
// SPDX-FileCopyrightText: Copyright (c) 2022-2024 NVIDIA CORPORATION & AFFILIATES.
|
|
//
|
|
//===----------------------------------------------------------------------===//
|
|
|
|
#include <cuda/experimental/__stf/stream/interfaces/slice_reduction_ops.cuh>
|
|
#include <cuda/experimental/__stf/stream/stream_ctx.cuh>
|
|
|
|
using namespace cuda::experimental::stf;
|
|
|
|
__global__ void add(int* ptr, int value)
|
|
{
|
|
*ptr = *ptr + value;
|
|
}
|
|
|
|
__global__ void check_val(const int* ptr, int expected)
|
|
{
|
|
assert(*ptr == expected);
|
|
}
|
|
|
|
/*
|
|
* This test ensures that we can reconstruct a piece of data where there are
|
|
* multiple "shared" instances, and "redux" instances
|
|
*/
|
|
int main()
|
|
{
|
|
stream_ctx ctx;
|
|
|
|
int ndevs;
|
|
cuda_safe_call(cudaGetDeviceCount(&ndevs));
|
|
|
|
if (ndevs < 2)
|
|
{
|
|
fprintf(stderr, "Skipping test: need at least 2 devices.\n");
|
|
return 0;
|
|
}
|
|
|
|
auto redux_op = std::make_shared<slice_reduction_op_sum<int>>();
|
|
|
|
int a = 17;
|
|
|
|
// init op (17)
|
|
auto handle = ctx.logical_data(make_slice(&a, 1));
|
|
|
|
// RW dev0 (18)
|
|
ctx.task(exec_place::device(0), handle.rw())->*[](auto stream, auto s) {
|
|
add<<<1, 1, 0, stream>>>(s.data_handle(), 1);
|
|
};
|
|
|
|
// READ dev1 (18)
|
|
ctx.task(exec_place::device(1), handle.read())->*[](auto stream, auto s) {
|
|
check_val<<<1, 1, 0, stream>>>(s.data_handle(), 18);
|
|
};
|
|
|
|
// REDUX dev1 (18 + 42)
|
|
ctx.task(exec_place::device(1), handle.relaxed(redux_op))->*[](auto stream, auto s) {
|
|
add<<<1, 1, 0, stream>>>(s.data_handle(), 42);
|
|
};
|
|
|
|
// READ dev0 (18 + 42)
|
|
ctx.task(exec_place::device(0), handle.read())->*[](auto stream, auto s) {
|
|
check_val<<<1, 1, 0, stream>>>(s.data_handle(), 18 + 42);
|
|
};
|
|
|
|
// READ
|
|
ctx.task(exec_place::host(), handle.read())->*[](auto stream, auto s) {
|
|
cuda_safe_call(cudaStreamSynchronize(stream));
|
|
EXPECT(s(0) == 18 + 42);
|
|
// printf("VALUE %d expected %d\n", s(0), 18 + 42);
|
|
};
|
|
|
|
ctx.finalize();
|
|
}
|